#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <propeller.h>
#include <dirent.h>
#include "simpletools.h"
#include "file_services.h"

#define SD_DO 22
#define SD_CLK 23
#define SD_DI 24
#define SD_CS 25

// 
#define MAX_FILES 100
#define MAX_DIRS 100

int file_count;
int dir_count;

char files[MAX_FILES][12];
char dirs[MAX_DIRS][12];

int init_fs()
{
    int ret;
    
    ret = sd_mount(SD_DO, SD_CLK, SD_DI, SD_CS);
    
    return ret;
}

int create_dir(char *path)
{
    int ret;
    
    ret = mkdir(path, 0);
    
    return ret;
}    

int delete_dir(char *path)
{
    int ret;
    
    ret = rmdir(path);
    
    return ret;
}

int delete_file(char *filename)
{
    int ret;
    
    ret = remove(filename);
    
    return ret;
}       

char *get_curr_dir(char *buf, int size)
{
    getcwd(buf, size);
    
    return buf;
}    

int set_curr_dir(char *path)
{
    int ret;

    ret = chdir(path);
    
    return ret;
}

char *path_add_slash(char *path)
{
    int path_length;
    char slash = '/';

    if(path)
    {
        path_length = strlen(path);

        if(path[path_length - 1] == slash)
        {
            return &path[path_length];
        }            
        else
        {
            path[path_length] = slash;
            path[path_length + 1] = '\0';

            return &path[path_length + 1];
        }
    }

    return NULL;
}

bool path_append(char *path, char *more)
{
    int path_length;
    char slash = '/';

    if(path && more)
    {
        path_length = strlen(path);


        if(path[path_length - 1] != slash)
        {
            path_add_slash(path);
        }            

        strcat(path, more);
    }

    return false;
}

void sort_files()
{
    bool incomplete = true;
    
    while(incomplete)
    {
        incomplete = false;

        for(int pos = 0; pos < file_count; pos++)
        {
            incomplete |= compare_swap_files(pos);
        }               
    }
}

bool compare_swap_files(int pos)
{
    char temp[12];
      
    int pos_first = pos;   
    int pos_next = pos + 1;

    if(strcmp(files[pos_first], files[pos_next]) > 0)  
    {
        strcpy(temp, files[pos_first]);
        memset(files[pos_first], 0, sizeof(files[pos_first]));      
        strcpy(files[pos_first], files[pos_next]);
        memset(files[pos_next], 0, sizeof(files[pos_next]));
        strcpy(files[pos_next], temp);      

        return true;   
    }   
   
    return false;
}

void sort_dirs()
{
    bool incomplete = true;
    
    while(incomplete)
    {
        incomplete = false;

        for(int pos = 0; pos < dir_count; pos++)
        {
            incomplete |= compare_swap_dirs(pos);
        }
    }
}

bool compare_swap_dirs(int pos)
{
    char temp[12];
      
    int pos_first = pos;   
    int pos_next = pos + 1;

    if(strcmp(dirs[pos_first], dirs[pos_next]) > 0)  
    {
        strcpy(temp, dirs[pos_first]);
        memset(dirs[pos_first], 0, sizeof(dirs[pos_first]));      
        strcpy(dirs[pos_first], dirs[pos_next]);
        memset(dirs[pos_next], 0, sizeof(dirs[pos_next]));
        strcpy(dirs[pos_next], temp);      

        return true;   
    }   
   
    return false;
}

int index_dir_entries(char *dir_path)
{
    file_count = -1;
    dir_count = -1;

    DIR *sd_dir;
    struct dirent *dir_entry;

    sd_dir = opendir(dir_path);

    if(sd_dir != NULL)
    {
        while((dir_entry = readdir(sd_dir)))
        {
            if(dir_entry->attr == ATTR_DIRECTORY)
            {
                dir_count++;
                
                strcpy(dirs[dir_count], dir_entry->d_name);
            }
            else
            {
                file_count++;
                
                strcpy(files[file_count], dir_entry->d_name);
            }                
        }        

        closedir(sd_dir);
        
        sort_dirs();
        sort_files();
    }
    else
    {
        print("Couldn't open the directory");
    }        

    return 0;
}

void print_dir_entries()
{
    for(int i = -1; i != dir_count; i++)
    {
        print("<DIR>%s\n", dirs[i + 1]);
    }
    
    for(int j = -1; j != file_count; j++)
    {
        print("%s\n", files[j + 1]);
    }
}    